import { JSONSchema7, JSONSchema7TypeName } from "json-schema";
import { DPMConfiguration } from "./main";
export declare enum CountPrecisionV030 {
    EXACT = "EXACT",
    APPROXIMATE = "APPROXIMATE",
    GREATER_THAN = "GREATER_THAN"
}
export interface StreamStatsV030 {
    byteCount?: number;
    byteCountPrecision?: CountPrecisionV030;
    inspectedCount: number;
    recordCount?: number;
    recordCountPrecision?: CountPrecisionV030;
}
export interface RegistryReferenceV030 {
    url: string;
    catalogSlug: string;
}
export interface StreamSetV030 {
    slug: string;
    configuration: DPMConfiguration;
    schemaTitles: string[];
    lastUpdateHash?: string;
    streamStats: StreamStatsV030;
}
export interface SourceV030 {
    type: string;
    slug: string;
    uris: string[];
    configuration?: DPMConfiguration;
    streamSets: StreamSetV030[];
}
export interface ValueTypeStatisticsV030 {
    valueType: JSONSchema7TypeName | "date";
    recordCount?: number;
    stringMaxLength?: number;
    stringMinLength?: number;
    numberMaxValue?: number;
    numberMinValue?: number;
    dateMaxValue?: Date;
    dateMinValue?: Date;
    stringOptions?: {
        [key: string]: number;
    };
}
export declare type PropertiesV030 = {
    [key: string]: SchemaV030;
};
export declare type ValueTypesV030 = {
    [key: string]: ValueTypeStatisticsV030;
};
export interface SchemaIdentifierV030 {
    registryUrl: string;
    catalogSlug: string;
    packageSlug: string;
    version: string;
    schemaTitle: string;
}
export interface DerivedFromV030 {
    displayName: string;
    url?: string;
    schemaIdentifier?: SchemaIdentifierV030;
}
export interface SchemaV030 extends JSONSchema7 {
    properties?: PropertiesV030;
    hidden?: boolean;
    unit?: string;
    recordsInspectedCount?: number;
    recordCount?: number;
    recordsNotPresent?: number;
    recordCountPrecision?: CountPrecisionV030;
    valueTypes?: ValueTypesV030;
    sampleRecords?: {
        [key: string]: unknown;
    }[];
    derivedFromDescription?: string;
    derivedFrom?: DerivedFromV030[];
}
export declare class PackageFileV030 {
    $schema: string;
    packageSlug: string;
    displayName: string;
    description: string;
    sources: SourceV030[];
    schemas: SchemaV030[];
    version: string;
    generatedBy: string;
    updatedDate: Date;
    readmeFile?: string;
    readmeMarkdown?: string;
    licenseFile?: string;
    licenseMarkdown?: string;
    website?: string;
    contactEmail?: string;
    registries?: RegistryReferenceV030[];
}
