import { UpdateMethod } from "./DataHandlingUtil";
import { DPMConfiguration } from "./main";
import { DPMPropertyTypes } from "./PackageUtil";
export declare const CURRENT_PACKAGE_FILE_SCHEMA_URL = "https://datapm.io/docs/package-file-schema-v0.31.5.json";
export declare enum CountPrecision {
    EXACT = "EXACT",
    APPROXIMATE = "APPROXIMATE",
    GREATER_THAN = "GREATER_THAN"
}
export declare enum PublishMethod {
    SCHEMA_ONLY = "SCHEMA_ONLY",
    SCHEMA_AND_DATA = "SCHEMA_AND_DATA",
    SCHEMA_PROXY_DATA = "SCHEMA_PROXY_DATA"
}
export interface StreamStats {
    byteCount?: number;
    byteCountPrecision?: CountPrecision;
    inspectedCount: number;
    recordCount?: number;
    recordCountPrecision?: CountPrecision;
}
export interface RegistryReference {
    url: string;
    catalogSlug: string;
    publishMethod: PublishMethod;
}
export interface StreamSet {
    slug: string;
    schemaTitles: string[];
    lastUpdateHash?: string;
    streamStats: StreamStats;
    updateMethods: UpdateMethod[];
}
export interface Source {
    type: string;
    slug: string;
    connectionConfiguration: DPMConfiguration;
    credentialsIdentifier?: string;
    configuration?: DPMConfiguration;
    streamSets: StreamSet[];
}
export interface ContentLabel {
    label: string;
    hidden: boolean;
    appliedByContentDetector?: string;
    ocurrenceCount?: number;
    valuesTestedCount?: number;
}
export interface ValueTypeStatistics {
    recordCount?: number;
    stringMaxLength?: number;
    stringMinLength?: number;
    numberMaxValue?: number;
    numberMinValue?: number;
    numberMaxPrecision?: number;
    numberMaxScale?: number;
    booleanTrueCount?: number;
    booleanFalseCount?: number;
    dateMaxValue?: Date;
    dateMinValue?: Date;
    dateMaxMillsecondsPrecision?: number;
    stringOptions?: {
        [key: string]: number;
    };
    contentLabels?: ContentLabel[];
    arrayTypes?: ValueTypes;
    arrayMinLength?: number;
    arrayMaxLength?: number;
    objectProperties?: Properties;
}
export declare type Properties = {
    [key: string]: Property;
};
export declare type ValueTypes = {
    [key in DPMPropertyTypes]?: ValueTypeStatistics;
};
export interface MajorVersionIdentifier {
    registryUrl: string;
    catalogSlug: string;
    packageSlug: string;
    majorVersion: number;
}
export interface SchemaIdentifier extends MajorVersionIdentifier {
    schemaTitle: string;
}
export interface DerivedFrom {
    displayName: string;
    url?: string;
    schemaIdentifier?: SchemaIdentifier;
}
export interface Schema {
    title: string;
    description?: string;
    properties: Properties;
    hidden?: boolean;
    recordsInspectedCount?: number;
    recordCount?: number;
    recordsNotPresent?: number;
    recordCountPrecision?: CountPrecision;
    sampleRecords?: {
        [key: string]: unknown;
    }[];
    derivedFromDescription?: string;
    derivedFrom?: DerivedFrom[];
    unit?: string;
}
export interface Property {
    title: string;
    description?: string;
    hidden?: boolean;
    unit?: string;
    types: ValueTypes;
}
export declare class PackageFile {
    $schema: string;
    canonical: boolean;
    modifiedProperties?: string[];
    packageSlug: string;
    displayName: string;
    description: string;
    sources: Source[];
    schemas: Schema[];
    version: string;
    generatedBy: string;
    updatedDate: Date;
    readmeFile?: string;
    readmeMarkdown?: string;
    licenseFile?: string;
    licenseMarkdown?: string;
    website?: string;
    contactEmail?: string;
    registries?: RegistryReference[];
}
