import { JSONSchema7, JSONSchema7TypeName } from "json-schema";
import { DPMConfiguration } from "./main";
export declare enum CountPrecision040 {
    EXACT = "EXACT",
    APPROXIMATE = "APPROXIMATE",
    GREATER_THAN = "GREATER_THAN"
}
export interface StreamStats040 {
    byteCount?: number;
    byteCountPrecision?: CountPrecision040;
    inspectedCount: number;
    recordCount?: number;
    recordCountPrecision?: CountPrecision040;
}
export interface RegistryReference040 {
    url: string;
    catalogSlug: string;
}
export interface StreamSet040 {
    slug: string;
    configuration: DPMConfiguration;
    schemaTitles: string[];
    lastUpdateHash?: string;
    streamStats: StreamStats040;
}
export interface Source040 {
    type: string;
    slug: string;
    uris: string[];
    configuration?: DPMConfiguration;
    streamSets: StreamSet040[];
}
export interface ContentLabel040 {
    label: string;
    hidden: boolean;
    appliedByContentDetector?: string;
    ocurrenceCount?: number;
    valuesTestedCount?: number;
}
export interface ValueTypeStatistics040 {
    valueType: JSONSchema7TypeName | "date";
    recordCount?: number;
    stringMaxLength?: number;
    stringMinLength?: number;
    numberMaxValue?: number;
    numberMinValue?: number;
    dateMaxValue?: Date;
    dateMinValue?: Date;
    stringOptions?: {
        [key: string]: number;
    };
    contentLabels?: ContentLabel040[];
}
export declare type Properties040 = {
    [key: string]: Schema040;
};
export declare type ValueTypes040 = {
    [key: string]: ValueTypeStatistics040;
};
export interface SchemaIdentifier040 {
    registryUrl: string;
    catalogSlug: string;
    packageSlug: string;
    version: string;
    schemaTitle: string;
}
export interface DerivedFrom040 {
    displayName: string;
    url?: string;
    schemaIdentifier?: SchemaIdentifier040;
}
export interface Schema040 extends JSONSchema7 {
    description?: string;
    properties?: Properties040;
    hidden?: boolean;
    unit?: string;
    recordsInspectedCount?: number;
    recordCount?: number;
    recordsNotPresent?: number;
    recordCountPrecision?: CountPrecision040;
    valueTypes?: ValueTypes040;
    sampleRecords?: {
        [key: string]: unknown;
    }[];
    derivedFromDescription?: string;
    derivedFrom?: DerivedFrom040[];
}
export declare class PackageFile040 {
    $schema: string;
    packageSlug: string;
    displayName: string;
    description: string;
    sources: Source040[];
    schemas: Schema040[];
    version: string;
    generatedBy: string;
    updatedDate: Date;
    readmeFile?: string;
    readmeMarkdown?: string;
    licenseFile?: string;
    licenseMarkdown?: string;
    website?: string;
    contactEmail?: string;
    registries?: RegistryReference040[];
}
