import { JSONSchema7, JSONSchema7TypeName } from "json-schema";
import { DPMConfiguration } from "./main";
export declare enum CountPrecision080 {
    EXACT = "EXACT",
    APPROXIMATE = "APPROXIMATE",
    GREATER_THAN = "GREATER_THAN"
}
export declare enum PublishMethod080 {
    SCHEMA_ONLY = "SCHEMA_ONLY",
    SCHEMA_AND_DATA = "SCHEMA_AND_DATA",
    SCHEMA_PROXY_DATA = "SCHEMA_PROXY_DATA"
}
export interface StreamStats080 {
    byteCount?: number;
    byteCountPrecision?: CountPrecision080;
    inspectedCount: number;
    recordCount?: number;
    recordCountPrecision?: CountPrecision080;
}
export interface RegistryReference080 {
    url: string;
    catalogSlug: string;
    publishMethod: PublishMethod080;
}
export interface StreamSet080 {
    slug: string;
    configuration: DPMConfiguration;
    schemaTitles: string[];
    lastUpdateHash?: string;
    streamStats: StreamStats080;
}
export interface Source080 {
    type: string;
    slug: string;
    connectionConfiguration: DPMConfiguration;
    credentialsIdentifier?: string;
    configuration?: DPMConfiguration;
    streamSets: StreamSet080[];
}
export interface ContentLabel080 {
    label: string;
    hidden: boolean;
    appliedByContentDetector?: string;
    ocurrenceCount?: number;
    valuesTestedCount?: number;
}
export interface ValueTypeStatistics080 {
    valueType: JSONSchema7TypeName | "date";
    recordCount?: number;
    stringMaxLength?: number;
    stringMinLength?: number;
    numberMaxValue?: number;
    numberMinValue?: number;
    dateMaxValue?: Date;
    dateMinValue?: Date;
    stringOptions?: {
        [key: string]: number;
    };
    contentLabels?: ContentLabel080[];
}
export declare type Properties080 = {
    [key: string]: Schema080;
};
export declare type ValueTypes080 = {
    [key: string]: ValueTypeStatistics080;
};
export interface MajorVersionIdentifier080 {
    registryUrl: string;
    catalogSlug: string;
    packageSlug: string;
    majorVersion: number;
}
export interface SchemaIdentifier080 extends MajorVersionIdentifier080 {
    schemaTitle: string;
}
export interface DerivedFrom080 {
    displayName: string;
    url?: string;
    schemaIdentifier?: SchemaIdentifier080;
}
export interface Schema080 extends JSONSchema7 {
    description?: string;
    properties?: Properties080;
    hidden?: boolean;
    unit?: string;
    recordsInspectedCount?: number;
    recordCount?: number;
    recordsNotPresent?: number;
    recordCountPrecision?: CountPrecision080;
    valueTypes?: ValueTypes080;
    sampleRecords?: {
        [key: string]: unknown;
    }[];
    derivedFromDescription?: string;
    derivedFrom?: DerivedFrom080[];
}
export declare class PackageFile080 {
    $schema: string;
    canonical: boolean;
    modifiedProperties?: string[];
    packageSlug: string;
    displayName: string;
    description: string;
    sources: Source080[];
    schemas: Schema080[];
    version: string;
    generatedBy: string;
    updatedDate: Date;
    readmeFile?: string;
    readmeMarkdown?: string;
    licenseFile?: string;
    licenseMarkdown?: string;
    website?: string;
    contactEmail?: string;
    registries?: RegistryReference080[];
}
