import { SemVer } from "semver";
import { PackageFile, Schema, Source, StreamSet, Properties } from "./PackageFile-v0.32.1";
import { UpdateMethod } from "./DataHandlingUtil";
export declare type DPMPropertyTypes = "string" | "number" | "integer" | "boolean" | "date" | "date-time" | "object" | "array" | "null";
export declare type DPMRecordValue = number | string | boolean | Date | bigint | {
    [key: string]: unknown;
} | Array<DPMRecordValue> | null;
export declare type DPMRecord = Record<string, DPMRecordValue>;
export declare type DPMConfiguration = Record<string, number | string | boolean | {
    [key: string]: unknown;
} | string[] | number[] | boolean[] | null>;
export interface RecordContext {
    schemaSlug: string;
    record: DPMRecord;
    offset?: number;
    receivedDate: Date;
}
export interface RecordStreamContext {
    sourceType: string;
    sourceSlug: string;
    streamSetSlug: string;
    streamSlug: string;
    recordContext: RecordContext;
    updateMethod: UpdateMethod;
}
export declare enum Compability {
    Identical = 0,
    MinorChange = 1,
    CompatibleChange = 2,
    BreakingChange = 3
}
export declare enum DifferenceType {
    REMOVE_SCHEMA = "REMOVE_SCHEMA",
    REMOVE_HIDDEN_SCHEMA = "REMOVE_HIDDEN_SCHEMA",
    ADD_SCHEMA = "ADD_SCHEMA",
    REMOVE_SOURCE = "REMOVE_SOURCE",
    CHANGE_PACKAGE_DISPLAY_NAME = "CHANGE_PACKAGE_DISPLAY_NAME",
    CHANGE_PACKAGE_DESCRIPTION = "CHANGE_PACKAGE_DESCRIPTION",
    CHANGE_SOURCE = "CHANGE_SOURCE",
    CHANGE_SOURCE_CONNECTION = "CHANGE_SOURCE_CONNECTION",
    CHANGE_SOURCE_CREDENTIALS = "CHANGE_SOURCE_CREDENTIALS",
    CHANGE_SOURCE_CONFIGURATION = "CHANGE_SOURCE_CONFIGURATION",
    CHANGE_STREAM_UPDATE_METHOD = "CHANGE_STREAM_UPDATE_METHOD",
    CHANGE_SOURCE_URIS = "CHANGE_SOURCE_URIS",
    CHANGE_STREAM_STATS = "CHANGE_STREAM_STATS",
    CHANGE_STREAM_UPDATE_HASH = "CHANGE_STREAM_UPDATE_HASH",
    ADD_PROPERTY = "ADD_PROPERTY",
    HIDE_PROPERTY = "HIDE_PROPERTY",
    UNHIDE_PROPERTY = "UNHIDE_PROPERTY",
    REMOVE_PROPERTY = "REMOVE_PROPERTY",
    REMOVE_HIDDEN_PROPERTY = "REMOVE_HIDDEN_PROPERTY",
    CHANGE_PROPERTY_TYPE = "CHANGE_PROPERTY_TYPE",
    CHANGE_PROPERTY_FORMAT = "CHANGE_PROPERTY_FORMAT",
    CHANGE_PROPERTY_UNIT = "CHANGE_PROPERTY_UNIT",
    CHANGE_PROPERTY_DESCRIPTION = "CHANGE_PROPERTY_DESCRIPTION",
    CHANGE_GENERATED_BY = "CHANGE_GENERATED_BY",
    CHANGE_UPDATED_DATE = "CHANGE_UPDATED_DATE",
    CHANGE_VERSION = "CHANGE_VERSION",
    CHANGE_README_MARKDOWN = "CHANGE_README_MARKDOWN",
    CHANGE_LICENSE_MARKDOWN = "CHANGE_LICENSE_MARKDOWN",
    CHANGE_README_FILE = "CHANGE_README_FILE",
    CHANGE_LICENSE_FILE = "CHANGE_LICENSE_FILE",
    CHANGE_WEBSITE = "CHANGE_WEBSITE",
    CHANGE_CONTACT_EMAIL = "CHANGE_CONTACT_EMAIL",
    REMOVE_STREAM_SET = "REMOVE_STREAM_SET"
}
export interface Difference {
    type: DifferenceType;
    pointer: string;
}
export interface Comparison {
    compatibility: Compability;
    differences: Difference[];
}
export declare function leastCompatible(a: Compability, b: Compability): Compability;
export declare function comparePackages(packageA: PackageFile, packageB: PackageFile): Difference[];
export declare function compareSchemas(priorSchemas: Schema[], newSchemas: Schema[]): Difference[];
export declare function compareSources(priorSources: Source[], newSources: Source[]): Difference[];
export declare function compareStream(priorStreamSet: StreamSet, newStreamSet: StreamSet, pointer?: string): Difference[];
export declare function compareSource(priorSource: Source, newSource: Source, pointer?: string): Difference[];
export declare function compareSchema(priorSchema: Schema, newSchema: Schema, pointer?: string): Difference[];
export declare function compareProperties(priorProperties: Properties, newProperties: Properties, pointer?: string): Difference[];
export declare function sourceURIsEquivalent(urisA: string[], urisB: string[]): boolean;
export declare function compareArrays(priorArray?: unknown[] | null, newArray?: unknown[] | null): boolean;
export declare function compareConfigObjects(priorObject?: DPMConfiguration | null, newObject?: DPMConfiguration | null): boolean;
export declare function diffCompatibility(diffs: Difference[]): Compability;
export declare function nextVersion(currentVersion: SemVer, diffCompatibility: Compability): SemVer;
export declare function compatibilityToString(compatibility: Compability): string;
export declare function loadPackageFileFromDisk(packageFilePath: string): PackageFile;
export declare function parsePackageFileJSONInBrowser(packageFileString: string): Promise<PackageFile>;
export declare function parsePackageFileJSON(packageFileString: string): PackageFile;
export declare function upgradePackageFile(packageFileObject: any): PackageFile;
export declare function validatePackageFileInBrowser(packageFile: PackageFile): Promise<void>;
export declare function getSchemaVersionFromPackageFile(packageFileObject: any): SemVer;
export declare function validatePackageFile(packageFile: PackageFile): void;
