import { DPMConfiguration } from "./PackageUtil";
export declare enum ParameterType {
    AutoComplete = "autocomplete",
    Confirm = "confirm",
    Text = "text",
    Select = "select",
    MultiSelect = "multiselect",
    AutoCompleteMultiSelect = "autocompleteMultiselect",
    Number = "number",
    Password = "password"
}
export interface ParameterOption {
    title: string;
    value?: string | boolean | number | unknown;
    disabled?: boolean;
    selected?: boolean;
}
export declare type ValueOrFunc<T extends string> = T;
export interface Parameter<T extends string = string> {
    type: ParameterType;
    name: ValueOrFunc<T>;
    defaultValue?: string | boolean | number;
    message: string;
    hint?: string;
    options?: ParameterOption[];
    numberMinimumValue?: number;
    numberMaximumValue?: number;
    stringMinimumLength?: number;
    stringMaximumLength?: number;
    multiSelectMinimumCount?: number;
    multiSelectMaximumCount?: number;
    allowFreeFormInput?: boolean;
    stringRegExp?: {
        pattern: RegExp;
        message: string;
    };
    onChange?: (input: string, currentOptions: ParameterOption[]) => Promise<ParameterOption[]>;
    validate?: (value: string[] | string | number | boolean, parameter: Parameter) => true | string;
    configuration: DPMConfiguration;
}
export declare type ParameterAnswer<T extends string> = {
    [id in T]: any;
};
