"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validPackageDisplayName = exports.validPackageDescription = exports.validateUsernameOrEmail = exports.emailAddressValid = exports.passwordValid = exports.collectionSlugValid = exports.packageSlugValid = exports.catalogSlugValid = exports.usernameValid = exports.PACKAGE_DISPLAY_NAME_MAX_LENGTH = exports.PACKAGE_DISPLAY_NAME_MIN_LENGTH = exports.PACKAGE_DESCRIPTION_MIN_LENGTH = exports.PACKAGE_DESCRIPTION_MAX_LENGTH = exports.CATALOG_SLUG_MAX_LENGTH = exports.PACKAGE_SLUG_MAX_LENGTH = exports.CATALOG_SLUG_REGEX = exports.COLLECTION_SLUG_REGEX = exports.PACKAGE_SLUG_REGEX = void 0;
exports.PACKAGE_SLUG_REGEX = /^[a-z0-9]+(?:(?:(?:[._]|__|[-]*)[a-z0-9]+)+)?$/;
exports.COLLECTION_SLUG_REGEX = /^[a-z\d](?:[a-z\d]|-(?=[a-z\d])){0,38}$/i;
exports.CATALOG_SLUG_REGEX = /^[a-z\d](?:[a-z\d]|-(?=[a-z\d])){0,38}$/i;
exports.PACKAGE_SLUG_MAX_LENGTH = 38;
exports.CATALOG_SLUG_MAX_LENGTH = 38;
exports.PACKAGE_DESCRIPTION_MAX_LENGTH = 256;
exports.PACKAGE_DESCRIPTION_MIN_LENGTH = 3;
exports.PACKAGE_DISPLAY_NAME_MIN_LENGTH = 3;
exports.PACKAGE_DISPLAY_NAME_MAX_LENGTH = 128;
function usernameValid(username) {
    const regex = /^[a-z\d](?:[a-z\d]|-(?=[a-z\d])){0,38}$/;
    if (username === undefined) {
        return "USERNAME_REQUIRED";
    }
    if (username.length === 0) {
        return "USERNAME_REQUIRED";
    }
    if (username.length > 39) {
        return "USERNAME_TOO_LONG";
    }
    if (username.toLowerCase().match(regex) == null) {
        return "INVALID_CHARACTERS";
    }
    return true;
}
exports.usernameValid = usernameValid;
function catalogSlugValid(slug) {
    if (slug === undefined)
        return `CATALOG_SLUG_REQUIRED`;
    if (slug.length === 0)
        return `CATALOG_SLUG_REQUIRED`;
    if (slug.length > exports.CATALOG_SLUG_MAX_LENGTH)
        return `CATALOG_SLUG_TOO_LONG`;
    if (!slug.match(exports.CATALOG_SLUG_REGEX))
        return `CATALOG_SLUG_INVALID`;
    return true;
}
exports.catalogSlugValid = catalogSlugValid;
function packageSlugValid(slug) {
    if (slug === undefined)
        return `PACKAGE_SLUG_REQUIRED`;
    if (slug.length === 0)
        return `PACKAGE_SLUG_REQUIRED`;
    if (slug.length > exports.PACKAGE_SLUG_MAX_LENGTH)
        return `PACKAGE_SLUG_TOO_LONG`;
    if (!slug.match(exports.PACKAGE_SLUG_REGEX))
        return "PACKAGE_SLUG_INVALID";
    return true;
}
exports.packageSlugValid = packageSlugValid;
function collectionSlugValid(slug) {
    if (slug === undefined)
        return `COLLECTION_SLUG_REQUIRED`;
    if (slug.length === 0)
        return `COLLECTION_SLUG_REQUIRED`;
    if (slug.length > 100)
        return `COLLECTION_SLUG_TOO_LONG`;
    if (!slug.match(exports.COLLECTION_SLUG_REGEX))
        return `COLLECTION_SLUG_INVALID`;
    return true;
}
exports.collectionSlugValid = collectionSlugValid;
function passwordValid(password) {
    const regex = /[0-9@#$%!]/;
    if (password === undefined || password.length === 0) {
        return "PASSWORD_REQUIRED";
    }
    if (password.length > 99) {
        return "PASSWORD_TOO_LONG";
    }
    if (password.length < 8) {
        return "PASSWORD_TOO_SHORT";
    }
    if (password.length < 16 && password.match(regex) == null) {
        return "INVALID_CHARACTERS";
    }
    return true;
}
exports.passwordValid = passwordValid;
function emailAddressValid(emailAddress) {
    const regex = /^(?=[A-Z0-9][A-Z0-9@._%+-]{5,253}$)[A-Z0-9._%+-]{1,64}@(?:(?=[A-Z0-9-]{1,63}\.)[A-Z0-9]+(?:-[A-Z0-9]+)*\.){1,8}[A-Z]{2,63}$/i;
    if (emailAddress == null)
        return `REQUIRED`;
    if (emailAddress.trim().length === 0)
        return `REQUIRED`;
    if (emailAddress.length > 254)
        return `TOO_LONG`;
    if (emailAddress.match(regex) == null)
        return "INVALID_EMAIL_ADDRESS_FORMAT";
    return true;
}
exports.emailAddressValid = emailAddressValid;
function validateUsernameOrEmail(value) {
    if ((value === null || value === void 0 ? void 0 : value.indexOf("@")) !== -1) {
        return emailAddressValid(value);
    }
    else {
        return usernameValid(value);
    }
}
exports.validateUsernameOrEmail = validateUsernameOrEmail;
function validPackageDescription(value) {
    if (value == null)
        return "REQUIRED";
    if (typeof value !== "string")
        return "MUST_BE_STRING";
    if (value.length < exports.PACKAGE_DESCRIPTION_MIN_LENGTH)
        return "TOO_SHORT";
    if (value.length > exports.PACKAGE_DESCRIPTION_MAX_LENGTH)
        return "TOO_LONG";
    return true;
}
exports.validPackageDescription = validPackageDescription;
function validPackageDisplayName(value) {
    if (value == null)
        return "REQUIRED";
    if (typeof value !== "string")
        return "MUST_BE_STRING";
    if (!value.match(/^[\w _-]+$/))
        return "INVALID_CHARACTERS";
    if (value.length < exports.PACKAGE_DISPLAY_NAME_MIN_LENGTH)
        return "TOO_SHORT";
    if (value.length > exports.PACKAGE_DISPLAY_NAME_MAX_LENGTH)
        return "TOO_LONG";
    return true;
}
exports.validPackageDisplayName = validPackageDisplayName;
//# sourceMappingURL=Validators.js.map