"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const PackageUtil_1 = require("../src/PackageUtil");
const main_1 = require("../src/main");
const semver_1 = require("semver");
const chai_1 = require("chai");
describe("Checking VersionUtil", () => {
    it("Compatibility ENUM Order", () => {
        (0, chai_1.expect)((0, PackageUtil_1.leastCompatible)(PackageUtil_1.Compability.Identical, PackageUtil_1.Compability.BreakingChange)).equal(PackageUtil_1.Compability.BreakingChange);
    });
    it("No change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = (0, PackageUtil_1.nextVersion)(oldVersion, PackageUtil_1.Compability.Identical);
        (0, chai_1.expect)(oldVersion.version).equal("1.0.3");
        (0, chai_1.expect)(newVersion.version).equal("1.0.3");
    });
    it("Minor change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = (0, PackageUtil_1.nextVersion)(oldVersion, PackageUtil_1.Compability.MinorChange);
        (0, chai_1.expect)(oldVersion.version).equal("1.0.3");
        (0, chai_1.expect)(newVersion.version).equal("1.0.4");
    });
    it("Compatible change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = (0, PackageUtil_1.nextVersion)(oldVersion, PackageUtil_1.Compability.CompatibleChange);
        (0, chai_1.expect)(oldVersion.version).equal("1.0.3");
        (0, chai_1.expect)(newVersion.version).equal("1.1.0");
    });
    it("Breaking change test", () => {
        const oldVersion = new semver_1.SemVer("1.0.3");
        const newVersion = (0, PackageUtil_1.nextVersion)(oldVersion, PackageUtil_1.Compability.BreakingChange);
        (0, chai_1.expect)(oldVersion.version).equal("1.0.3");
        (0, chai_1.expect)(newVersion.version).equal("2.0.0");
    });
    it("Simple property schema comparison", () => {
        const propertiesA1 = {
            propertyA: {
                title: "propertyA",
                types: {
                    string: {}
                }
            }
        };
        const propertiesA2 = {
            propertyA: {
                title: "propertyA",
                types: {
                    string: {}
                }
            }
        };
        (0, chai_1.expect)((0, PackageUtil_1.compareProperties)(propertiesA1, propertiesA2).length).equal(0);
        propertiesA2.propertyA.types.date = {};
        const changeType = (0, PackageUtil_1.compareProperties)(propertiesA1, propertiesA2);
        (0, chai_1.expect)(changeType[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_PROPERTY_TYPE);
        (0, chai_1.expect)(changeType[0].pointer).equal("#/properties/propertyA");
    });
    it("Type arrays vs singluar values", () => {
        const schemaA1 = {
            title: "SchemaA",
            properties: {
                string: {
                    title: "string",
                    types: {
                        string: {}
                    }
                },
                number: {
                    title: "number",
                    types: {
                        number: {}
                    }
                }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            properties: {
                string: {
                    title: "string",
                    types: {
                        string: {}
                    }
                },
                number: {
                    title: "number",
                    types: {
                        number: {}
                    }
                }
            }
        };
        const diff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(diff.length).equal(0);
        schemaA1.properties.string.types.number = {};
        const arrayVsNotDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(arrayVsNotDiff.length).equal(1);
        (0, chai_1.expect)(arrayVsNotDiff[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_PROPERTY_TYPE);
        schemaA2.properties.string.types.number = {};
        const equalDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(equalDiff.length).equal(0);
    });
    it("Schema comparison", () => {
        const schemaA1 = {
            title: "SchemaA",
            properties: {
                string: { title: "string", types: { string: {} } },
                number: { title: "number", types: { number: {} } }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            properties: {
                string: { title: "string", types: { string: {} } },
                number: { title: "number", types: { number: {} } }
            }
        };
        const diff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(diff.length).equal(0);
        schemaA2.properties.boolean = { title: "boolean", types: { boolean: {} } };
        const compatibleChange = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(compatibleChange.length).equal(1);
        (0, chai_1.expect)(compatibleChange[0].type).equal(PackageUtil_1.DifferenceType.ADD_PROPERTY);
        schemaA1.properties.date = {
            title: "date",
            types: { date: {} }
        };
        const addPropertyDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(addPropertyDiff.length).equal(2);
        const propertyRemoved = addPropertyDiff.find((d) => d.type === PackageUtil_1.DifferenceType.ADD_PROPERTY);
        (0, chai_1.expect)(propertyRemoved != null).equal(true);
        schemaA1.properties.boolean = { title: "boolean", types: { boolean: {} } };
        schemaA2.properties.date = {
            title: "date",
            types: { date: {} }
        };
        const finalDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(finalDiff).length(0);
    });
    it("Diff compatibility testing", () => {
        (0, chai_1.expect)((0, PackageUtil_1.nextVersion)(new semver_1.SemVer("1.0.2"), PackageUtil_1.Compability.BreakingChange).version).equal(new semver_1.SemVer("2.0.0").version);
        (0, chai_1.expect)((0, PackageUtil_1.nextVersion)(new semver_1.SemVer("1.0.3"), PackageUtil_1.Compability.CompatibleChange).version).equal(new semver_1.SemVer("1.1.0").version);
        (0, chai_1.expect)((0, PackageUtil_1.nextVersion)(new semver_1.SemVer("1.0.3"), PackageUtil_1.Compability.MinorChange).version).equal(new semver_1.SemVer("1.0.4").version);
    });
    it("Nested Objects Comparison", () => {
        var _a, _b, _c, _d, _e, _f;
        const schemaA1 = {
            title: "SchemaA",
            properties: {
                object: {
                    title: "object",
                    types: {
                        object: {
                            objectProperties: {
                                string1: { title: "string1", types: { string: {} } }
                            }
                        }
                    }
                },
                number: { title: "number", types: { number: {} } }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            properties: {
                object: {
                    title: "object",
                    types: {
                        object: {
                            objectProperties: {
                                string1: { title: "string1", types: { string: {} } }
                            }
                        }
                    }
                },
                number: { title: "number", types: { number: {} } }
            }
        };
        const firstDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(firstDiff).length(0);
        (0, chai_1.expect)((0, PackageUtil_1.diffCompatibility)(firstDiff)).equal(PackageUtil_1.Compability.Identical);
        ((_a = schemaA2.properties.object.types.object) === null || _a === void 0 ? void 0 : _a.objectProperties).string2 = {
            title: "string2",
            types: { string: {} }
        };
        const compatibleDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(compatibleDiff).length(1);
        (0, chai_1.expect)(compatibleDiff[0].type).equal(PackageUtil_1.DifferenceType.ADD_PROPERTY);
        (0, chai_1.expect)(compatibleDiff[0].pointer).equal("#/SchemaA/properties/object/properties/string2");
        const compatibleComparison = (0, PackageUtil_1.diffCompatibility)(compatibleDiff);
        (0, chai_1.expect)(compatibleComparison).equal(PackageUtil_1.Compability.CompatibleChange);
        ((_b = schemaA1.properties.object.types.object) === null || _b === void 0 ? void 0 : _b.objectProperties).string3 = {
            title: "string3",
            types: { string: {} }
        };
        ((_c = schemaA1.properties.object.types.object) === null || _c === void 0 ? void 0 : _c.objectProperties).string4 = {
            title: "string4",
            types: { string: {} }
        };
        const breakingDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(breakingDiff).length(3);
        (0, chai_1.expect)(breakingDiff[0].type).equal(PackageUtil_1.DifferenceType.REMOVE_PROPERTY);
        (0, chai_1.expect)(breakingDiff[0].pointer).equal("#/SchemaA/properties/object/properties/string3");
        (0, chai_1.expect)(breakingDiff[1].type).equal(PackageUtil_1.DifferenceType.REMOVE_PROPERTY);
        (0, chai_1.expect)(breakingDiff[1].pointer).equal("#/SchemaA/properties/object/properties/string4");
        (0, chai_1.expect)(breakingDiff[2].type).equal(PackageUtil_1.DifferenceType.ADD_PROPERTY);
        (0, chai_1.expect)(breakingDiff[2].pointer).equal("#/SchemaA/properties/object/properties/string2");
        const breakingChange = (0, PackageUtil_1.diffCompatibility)(breakingDiff);
        (0, chai_1.expect)(breakingChange).equal(PackageUtil_1.Compability.BreakingChange);
        ((_d = schemaA1.properties.object.types.object) === null || _d === void 0 ? void 0 : _d.objectProperties).string2 = {
            title: "string2",
            types: { string: {} }
        };
        ((_e = schemaA2.properties.object.types.object) === null || _e === void 0 ? void 0 : _e.objectProperties).string3 = {
            title: "string2",
            types: { string: {} }
        };
        ((_f = schemaA2.properties.object.types.object) === null || _f === void 0 ? void 0 : _f.objectProperties).string4 = {
            title: "string3",
            types: { string: {} }
        };
        const finalDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(finalDiff).length(0);
        (0, chai_1.expect)((0, PackageUtil_1.diffCompatibility)(finalDiff)).equal(PackageUtil_1.Compability.Identical);
    });
    it("Catalog slug validation", () => {
        (0, chai_1.expect)((0, main_1.catalogSlugValid)("a")).equal(true);
        (0, chai_1.expect)((0, main_1.catalogSlugValid)("0")).equal(true);
        (0, chai_1.expect)((0, main_1.catalogSlugValid)("a-b")).equal(true);
        (0, chai_1.expect)((0, main_1.catalogSlugValid)("a-b-123")).equal(true);
        (0, chai_1.expect)((0, main_1.catalogSlugValid)("a".repeat(39))).equal("CATALOG_SLUG_TOO_LONG");
        (0, chai_1.expect)((0, main_1.catalogSlugValid)(undefined)).equal("CATALOG_SLUG_REQUIRED");
        (0, chai_1.expect)((0, main_1.catalogSlugValid)("")).equal("CATALOG_SLUG_REQUIRED");
        (0, chai_1.expect)((0, main_1.catalogSlugValid)("a_b")).equal("CATALOG_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.catalogSlugValid)("a--b")).equal("CATALOG_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.catalogSlugValid)("a-b-")).equal("CATALOG_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.catalogSlugValid)("-a-b")).equal("CATALOG_SLUG_INVALID");
    });
    it("Package slug validation", () => {
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a")).equal(true);
        (0, chai_1.expect)((0, main_1.packageSlugValid)("0")).equal(true);
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a.b")).equal(true);
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a--b")).equal(true);
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a__b")).equal(true);
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a__b----c.123")).equal(true);
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a".repeat(100))).equal("PACKAGE_SLUG_TOO_LONG");
        (0, chai_1.expect)((0, main_1.packageSlugValid)(undefined)).equal("PACKAGE_SLUG_REQUIRED");
        (0, chai_1.expect)((0, main_1.packageSlugValid)("")).equal("PACKAGE_SLUG_REQUIRED");
        (0, chai_1.expect)((0, main_1.packageSlugValid)(".")).equal("PACKAGE_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.packageSlugValid)("-")).equal("PACKAGE_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.packageSlugValid)("_")).equal("PACKAGE_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a@b")).equal("PACKAGE_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a.")).equal("PACKAGE_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a..b")).equal("PACKAGE_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a-")).equal("PACKAGE_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a_")).equal("PACKAGE_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.packageSlugValid)("a___c")).equal("PACKAGE_SLUG_INVALID");
    });
    it("Collection slug validation", () => {
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a")).equal(true);
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a--b")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a__b")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a__b----c.123")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a".repeat(101))).equal("COLLECTION_SLUG_TOO_LONG");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)(undefined)).equal("COLLECTION_SLUG_REQUIRED");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a.b")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("")).equal("COLLECTION_SLUG_REQUIRED");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("0")).equal(true);
        (0, chai_1.expect)((0, main_1.collectionSlugValid)(".")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("-")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("_")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a@b")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a.")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a..b")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a-")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a_")).equal("COLLECTION_SLUG_INVALID");
        (0, chai_1.expect)((0, main_1.collectionSlugValid)("a___c")).equal("COLLECTION_SLUG_INVALID");
    });
    it("Compare source objects", () => {
        const sourceA = {
            slug: "datapm",
            type: "test",
            connectionConfiguration: {},
            configuration: {
                uris: ["http://datapm.io/test", "http://datapm.io/test2"]
            },
            streamSets: [
                {
                    slug: "streamA",
                    lastUpdateHash: "abc123",
                    schemaTitles: ["A"],
                    streamStats: {
                        inspectedCount: 0
                    },
                    updateMethods: [],
                    endReached: false
                }
            ]
        };
        const sourceB = {
            slug: "datapm",
            type: "test",
            connectionConfiguration: {},
            configuration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
            streamSets: [
                {
                    slug: "streamA",
                    lastUpdateHash: "abc123",
                    schemaTitles: ["A"],
                    streamStats: {
                        inspectedCount: 0
                    },
                    updateMethods: [],
                    endReached: false
                }
            ]
        };
        const diffs = (0, main_1.compareSource)(sourceA, sourceB);
        (0, chai_1.expect)(diffs.length).equals(0);
        sourceA.streamSets[0].lastUpdateHash = "test";
        const diffs2 = (0, main_1.compareSource)(sourceA, sourceB);
        (0, chai_1.expect)(diffs2[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_STREAM_UPDATE_HASH);
        sourceA.streamSets[0].lastUpdateHash = sourceB.streamSets[0].lastUpdateHash;
        (0, chai_1.expect)(sourceA.configuration != null).eq(true);
        sourceA.configuration.uris = ["http://datapm.io.test"];
        const diffs3 = (0, main_1.compareSource)(sourceA, sourceB);
        (0, chai_1.expect)(diffs3[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_SOURCE_CONFIGURATION);
    });
    it("Compare source arrays", () => {
        const sourceA = [
            {
                slug: "datapm",
                type: "test",
                connectionConfiguration: {},
                configuration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
                streamSets: [
                    {
                        slug: "streamA",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 0
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const sourceB = [
            {
                slug: "datapm",
                type: "test",
                connectionConfiguration: {},
                configuration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
                streamSets: [
                    {
                        slug: "streamA",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 0
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const diffs = (0, main_1.compareSources)(sourceA, sourceB);
        (0, chai_1.expect)(diffs.length).equals(0);
        sourceA[0].configuration.uris = ["test"];
        const diffs2 = (0, main_1.compareSources)(sourceA, sourceB);
        (0, chai_1.expect)(diffs2[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_SOURCE_CONFIGURATION);
    });
    it("Detected removed sources", () => {
        const sourceA = [
            {
                slug: "datapm",
                type: "test",
                connectionConfiguration: {},
                configuration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
                streamSets: [
                    {
                        slug: "streamA",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 0
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const sourceB = [
            {
                slug: "datapm2",
                type: "test",
                connectionConfiguration: {},
                configuration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
                streamSets: [
                    {
                        slug: "streamA",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 0
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const diffs = (0, main_1.compareSources)(sourceA, sourceB);
        (0, chai_1.expect)(diffs.length).equals(1);
        (0, chai_1.expect)(diffs[0].type).equals(PackageUtil_1.DifferenceType.REMOVE_SOURCE);
    });
    it("Stream status change detection", () => {
        const sourceA = [
            {
                slug: "datapm",
                type: "test",
                connectionConfiguration: {},
                configuration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
                streamSets: [
                    {
                        slug: "test",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 0
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const sourceB = [
            {
                slug: "datapm",
                type: "test",
                connectionConfiguration: {},
                configuration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
                streamSets: [
                    {
                        slug: "test",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 1
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const diffs = (0, main_1.compareSources)(sourceA, sourceB);
        (0, chai_1.expect)(diffs.length).equals(1);
        (0, chai_1.expect)(diffs[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_STREAM_STATS);
        sourceB[0].streamSets[0].slug += "!";
        const diffs2 = (0, main_1.compareSources)(sourceA, sourceB);
        (0, chai_1.expect)(diffs2.length).equals(1);
        (0, chai_1.expect)(diffs2[0].type).equal(PackageUtil_1.DifferenceType.REMOVE_STREAM_SET);
    });
    it("Source connection detection", () => {
        const sourceA = [
            {
                slug: "datapm",
                type: "test",
                connectionConfiguration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
                configuration: {},
                streamSets: [
                    {
                        slug: "test",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 1
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const sourceB = [
            {
                slug: "datapm",
                type: "test",
                connectionConfiguration: {
                    uris: ["http://datapm.io/test", "http://datapm.io/test2"],
                    newValue: "a"
                },
                configuration: {},
                streamSets: [
                    {
                        slug: "test",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 1
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const diffs = (0, main_1.compareSources)(sourceA, sourceB);
        (0, chai_1.expect)(diffs.length).equals(1);
        (0, chai_1.expect)(diffs[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_SOURCE_CONNECTION);
    });
    it("Source credentials detection", () => {
        const sourceA = [
            {
                slug: "datapm",
                type: "test",
                credentialsIdentifier: "test",
                connectionConfiguration: {},
                configuration: {},
                streamSets: [
                    {
                        slug: "test",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 1
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const sourceB = [
            {
                slug: "datapm",
                type: "test",
                credentialsIdentifier: "test2",
                connectionConfiguration: {},
                configuration: {},
                streamSets: [
                    {
                        slug: "test",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 1
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const diffs = (0, main_1.compareSources)(sourceA, sourceB);
        (0, chai_1.expect)(diffs.length).equals(1);
        (0, chai_1.expect)(diffs[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_SOURCE_CREDENTIALS);
    });
    it("Source configuration detection", () => {
        const sourceA = [
            {
                slug: "datapm",
                type: "test",
                connectionConfiguration: {},
                configuration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
                streamSets: [
                    {
                        slug: "test",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 1
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const sourceB = [
            {
                slug: "datapm",
                type: "test",
                configuration: {
                    uris: ["http://datapm.io/test", "http://datapm.io/test2"],
                    newValue: "a"
                },
                connectionConfiguration: {},
                streamSets: [
                    {
                        slug: "test",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 1
                        },
                        updateMethods: [],
                        endReached: false
                    }
                ]
            }
        ];
        const diffs = (0, main_1.compareSources)(sourceA, sourceB);
        (0, chai_1.expect)(diffs.length).equals(1);
        (0, chai_1.expect)(diffs[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_SOURCE_CONFIGURATION);
    });
    it("Source configuration detection", () => {
        const sourceA = [
            {
                slug: "datapm",
                type: "test",
                connectionConfiguration: {},
                configuration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
                streamSets: [
                    {
                        slug: "test",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 1
                        },
                        updateMethods: [main_1.UpdateMethod.BATCH_FULL_SET],
                        endReached: false
                    }
                ]
            }
        ];
        const sourceB = [
            {
                slug: "datapm",
                type: "test",
                configuration: { uris: ["http://datapm.io/test", "http://datapm.io/test2"] },
                connectionConfiguration: {},
                streamSets: [
                    {
                        slug: "test",
                        lastUpdateHash: "abc123",
                        schemaTitles: ["A"],
                        streamStats: {
                            inspectedCount: 1
                        },
                        updateMethods: [main_1.UpdateMethod.APPEND_ONLY_LOG],
                        endReached: false
                    }
                ]
            }
        ];
        const diffs = (0, main_1.compareSources)(sourceA, sourceB);
        (0, chai_1.expect)(diffs.length).equals(1);
        (0, chai_1.expect)(diffs[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_STREAM_UPDATE_METHOD);
    });
    it("Package File updated dates", function () {
        const packageFileA = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test",
            sources: []
        };
        const packageFileB = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test",
            sources: []
        };
        (0, chai_1.expect)((0, PackageUtil_1.comparePackages)(packageFileA, packageFileB).some((d) => d.type === "CHANGE_UPDATED_DATE")).equal(false);
        packageFileB.updatedDate = new Date(new Date().getTime() - 100);
        const diff = (0, PackageUtil_1.comparePackages)(packageFileA, packageFileB);
        (0, chai_1.expect)(diff.some((d) => d.type === "CHANGE_UPDATED_DATE")).equal(true);
    });
    it("Package File updated versions", function () {
        const packageFileA = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            sources: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test"
        };
        const packageFileB = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            sources: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test"
        };
        (0, chai_1.expect)((0, PackageUtil_1.comparePackages)(packageFileA, packageFileB).some((d) => d.type === "CHANGE_VERSION")).equal(false);
        packageFileB.version = "1.0.1";
        const diff = (0, PackageUtil_1.comparePackages)(packageFileA, packageFileB);
        (0, chai_1.expect)(diff.some((d) => d.type === "CHANGE_VERSION")).equal(true);
    });
    it("Package File updated readme", function () {
        const packageFileA = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            sources: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test",
            readmeMarkdown: "Some readme content"
        };
        const packageFileB = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            sources: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test",
            readmeMarkdown: packageFileA.readmeMarkdown
        };
        (0, chai_1.expect)((0, PackageUtil_1.comparePackages)(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_README_MARKDOWN)).equal(false);
        packageFileB.readmeFile = "some-new-file.README.md";
        (0, chai_1.expect)((0, PackageUtil_1.comparePackages)(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_README_FILE)).equal(true);
        packageFileB.readmeMarkdown = "other content";
        const diff = (0, PackageUtil_1.comparePackages)(packageFileA, packageFileB);
        (0, chai_1.expect)(diff.some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_README_MARKDOWN)).equal(true);
    });
    it("Package File updated license", function () {
        const packageFileA = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            sources: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test",
            licenseMarkdown: "Some content"
        };
        const packageFileB = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            sources: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test",
            licenseMarkdown: packageFileA.licenseMarkdown
        };
        (0, chai_1.expect)((0, PackageUtil_1.comparePackages)(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_LICENSE_MARKDOWN)).equal(false);
        packageFileB.licenseFile = "some-new-file.LICENSE.md";
        (0, chai_1.expect)((0, PackageUtil_1.comparePackages)(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_LICENSE_FILE)).equal(true);
        packageFileB.licenseMarkdown = "other content";
        const diff = (0, PackageUtil_1.comparePackages)(packageFileA, packageFileB);
        (0, chai_1.expect)(diff.some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_LICENSE_MARKDOWN)).equal(true);
    });
    it("Package File updated contact email", function () {
        const packageFileA = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            sources: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test",
            readmeMarkdown: "Some readme content",
            contactEmail: "test@test.com"
        };
        const packageFileB = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            sources: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test",
            readmeMarkdown: packageFileA.readmeMarkdown,
            contactEmail: packageFileA.contactEmail
        };
        (0, chai_1.expect)((0, PackageUtil_1.comparePackages)(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_CONTACT_EMAIL)).equal(false);
        packageFileB.contactEmail = "testb@test.com";
        (0, chai_1.expect)((0, PackageUtil_1.comparePackages)(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_CONTACT_EMAIL)).equal(true);
    });
    it("Package File updated website", function () {
        const packageFileA = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            sources: [],
            version: "1.0.0",
            updatedDate: new Date(),
            description: "Back test",
            readmeMarkdown: "Some readme content",
            contactEmail: "test@test.com",
            website: "https://dreamingwell.com"
        };
        const packageFileB = {
            ...new main_1.PackageFile(),
            packageSlug: "test",
            displayName: "test",
            generatedBy: "test",
            schemas: [],
            sources: [],
            version: "1.0.0",
            updatedDate: packageFileA.updatedDate,
            description: "Back test",
            readmeMarkdown: packageFileA.readmeMarkdown,
            contactEmail: packageFileA.contactEmail,
            website: packageFileA.website
        };
        (0, chai_1.expect)((0, PackageUtil_1.comparePackages)(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_WEBSITE)).equal(false);
        packageFileB.website = "https://datapm.io";
        (0, chai_1.expect)((0, PackageUtil_1.comparePackages)(packageFileA, packageFileB).some((d) => d.type === PackageUtil_1.DifferenceType.CHANGE_WEBSITE)).equal(true);
    });
    it("Compare hidden properties", () => {
        const schemaA1 = {
            title: "SchemaA",
            properties: {
                string: { title: "string", types: { string: {} } },
                number: { title: "number", types: { number: {} } }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            properties: {
                string: { title: "string", types: { string: {} } },
                number: { title: "number", types: { number: {} } }
            }
        };
        const diff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(diff.length).equal(0);
        schemaA2.properties.string.hidden = true;
        const propertyHiddenDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(propertyHiddenDiff.length).equal(1);
        (0, chai_1.expect)(propertyHiddenDiff[0].type).equal(PackageUtil_1.DifferenceType.HIDE_PROPERTY);
        const propertyHiddenCompatibility = (0, PackageUtil_1.diffCompatibility)(propertyHiddenDiff);
        (0, chai_1.expect)(propertyHiddenCompatibility).equal(PackageUtil_1.Compability.BreakingChange);
        schemaA1.properties.string.hidden = true;
        delete schemaA2.properties.string;
        const removeHiddenPropertyDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(removeHiddenPropertyDiff.length).equal(1);
        (0, chai_1.expect)(removeHiddenPropertyDiff[0].type).equal(PackageUtil_1.DifferenceType.REMOVE_HIDDEN_PROPERTY);
        const removeHiddenPropertyCompatibility = (0, PackageUtil_1.diffCompatibility)(removeHiddenPropertyDiff);
        (0, chai_1.expect)(removeHiddenPropertyCompatibility).equal(PackageUtil_1.Compability.MinorChange);
    });
    it("Compare changing property unit", () => {
        const schemaA1 = {
            title: "SchemaA",
            properties: {
                string: { title: "string", types: { string: {} } },
                number: { title: "number", types: { number: {} }, unit: "something" }
            }
        };
        const schemaA2 = {
            title: "SchemaA",
            properties: {
                string: { title: "string", types: { string: {} } },
                number: { title: "number", types: { number: {} }, unit: "something" }
            }
        };
        const diff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(diff.length).equal(0);
        schemaA2.properties.number.unit = "something else";
        const propertyHiddenDiff = (0, PackageUtil_1.compareSchema)(schemaA1, schemaA2);
        (0, chai_1.expect)(propertyHiddenDiff.length).equal(1);
        (0, chai_1.expect)(propertyHiddenDiff[0].type).equal(PackageUtil_1.DifferenceType.CHANGE_PROPERTY_UNIT);
        const propertyHiddenCompatibility = (0, PackageUtil_1.diffCompatibility)(propertyHiddenDiff);
        (0, chai_1.expect)(propertyHiddenCompatibility).equal(PackageUtil_1.Compability.MinorChange);
    });
});
//# sourceMappingURL=PackageComparison.spec.js.map