"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const main_1 = require("../src/main");
const chai_1 = require("chai");
const fs_1 = __importDefault(require("fs"));
describe("PackageFile checks", () => {
    it("Should have correct schema value", function () {
        const test = new main_1.PackageFile();
        (0, chai_1.expect)(test.$schema).equal("https://datapm.io/docs/package-file-schema-v0.32.1.json");
    });
    it("Should parse dates", async function () {
        const packageFileString = fs_1.default.readFileSync("test/packageFiles/congressional-legislators.datapm.json");
        const packageFile = (0, main_1.parsePackageFileJSON)(packageFileString.toString());
        (0, chai_1.expect)(packageFile.updatedDate.getTime()).equals(1601560469442);
    });
    it("Should load readme and license files", async function () {
        const packageFile = (0, main_1.loadPackageFileFromDisk)("test/packageFiles/congressional-legislators.datapm.json");
        (0, chai_1.expect)(packageFile.updatedDate.getTime()).equals(1601560469442);
        (0, chai_1.expect)(packageFile.readmeMarkdown).contains("This is where a readme might go.");
        (0, chai_1.expect)(packageFile.licenseMarkdown).contains("This is not a real license. Just a test.");
    });
    it("Should have v0.3.0 sources array separate from schemas", async function () {
        const packageFile = (0, main_1.loadPackageFileFromDisk)("test/packageFiles/congressional-legislators.datapm.json");
        (0, chai_1.expect)(packageFile.sources.length).equal(1);
        (0, chai_1.expect)(packageFile.sources[0].configuration.uris.length).equal(1);
        (0, chai_1.expect)(packageFile.sources[0].configuration.uris[0]).equal("https://theunitedstates.io/congress-legislators/legislators-current.csv");
    });
    it("Should have v0.4.0 max and min number values from schemas", async function () {
        var _a, _b;
        const packageFile = (0, main_1.loadPackageFileFromDisk)("test/packageFiles/congressional-legislators.datapm.json");
        (0, chai_1.expect)(packageFile.schemas.length).equal(1);
        (0, chai_1.expect)(Object.values(packageFile.schemas[0].properties.district.types).find((a) => typeof a.numberMaxValue === "string")).equal(undefined);
        (0, chai_1.expect)(Object.values(packageFile.schemas[0].properties.district.types).find((a) => typeof a.numberMinValue === "string")).equal(undefined);
        (0, chai_1.expect)(((_a = packageFile.schemas[0].properties.district.types.number) === null || _a === void 0 ? void 0 : _a.numberMaxValue) === 9);
        (0, chai_1.expect)(((_b = packageFile.schemas[0].properties.district.types.number) === null || _b === void 0 ? void 0 : _b.numberMaxValue) === 0);
    });
    it("Should throw invalid package file error", async function () {
        const packageFile = (0, main_1.loadPackageFileFromDisk)("test/packageFiles/congressional-legislators.datapm.json");
        packageFile.packageSlug += "-";
        let errorFound = false;
        try {
            (0, main_1.validatePackageFile)(packageFile);
        }
        catch (error) {
            errorFound = true;
        }
        (0, chai_1.expect)(errorFound).equal(true);
    });
    it("Should have v0.8.0 canonical value true", async function () {
        const packageFile = (0, main_1.loadPackageFileFromDisk)("test/packageFiles/congressional-legislators.datapm.json");
        (0, chai_1.expect)(packageFile.canonical).equal(true);
    });
    it("Should parse heirarchical object schema", async function () {
        var _a, _b, _c, _d, _e;
        const packageFile = (0, main_1.loadPackageFileFromDisk)("test/packageFiles/object-test.datapm.json");
        (0, chai_1.expect)((_e = (_d = (_c = (_b = (_a = packageFile.schemas[0].properties) === null || _a === void 0 ? void 0 : _a.testObject.types) === null || _b === void 0 ? void 0 : _b.object) === null || _c === void 0 ? void 0 : _c.objectProperties) === null || _d === void 0 ? void 0 : _d.subObject.types.object) === null || _e === void 0 ? void 0 : _e.recordCount).equal(1);
    });
});
//# sourceMappingURL=PackageFileParse.spec.js.map